

//-------------------------------------------------------------------------
//-------------Bibliotheken einbinden--------------------------------------
//-------------------------------------------------------------------------

#include <avr/interrupt.h>
#include <avr/io.h>
#include <util/delay.h>
#include <util/twi.h>
#include <stdlib.h>
#include <stdint.h>
#include <stdio.h>
#include <math.h>
#include <avr/wdt.h>
#include <string.h>	

#include "lcd-routines.h"

//------------------------------------------------------------------------
//------------Prozessortakt-einstellen------------------------------------
//------------------------------------------------------------------------
#ifndef F_CPU             	   
#define F_CPU 4000000 UL          
#endif

#define TWIM_LESEN       1
#define TWIM_SCHREIBEN   0

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

//------------------------------------------------------------------------------------------------------------//
//-----------------Variablen-deklartion-----------------------------------------------------------------------// 
//------------------------------------------------------------------------------------------------------------//
volatile uint32_t Si_wert	= 0;
volatile uint16_t Overflow  = 0; 		// Anzahl der Timer Overflows die waehrend der Messperiode passieren 		
volatile uint16_t Starttime = 0;
volatile uint16_t Endtime 	= 0;
volatile uint16_t Durchlauf;   		// Job Flag
volatile uint16_t sekunden 	= 0;

uint32_t Zeit_ti 	= 	0;
uint32_t Zeit1_ti	=	0;
uint32_t Zeit2_ti	=	0;
uint32_t ti_neu	 	= 	0;
uint32_t ti1_neu 	= 	0;
uint32_t ti2_neu 	= 	0;
uint32_t ti_rest 	= 	0; 
uint32_t Ct		 	= 	21000; 
uint32_t Ges_Zeit	= 	60;
uint32_t Xi		 	=	0;
uint32_t Xi1	 	=	0;
uint32_t Xi2		= 	0;
uint32_t Xi_rest	=	0;

uint32_t Geschwindigkeit_in_10min;
uint32_t Standardabweichung;

char lcd_Geschwindigkeit_String[11];
char lcd_Standardabweichung_String[11];
char lcd_Impulse_String[10];
char lcd_sekunden_String[10];
char lcd_Overflow_String[10];
char my_string[7];

//static unsigned char ErsteFlanke = TRUE;
//-------------------------------------------------------------------------------------------------------------//
//--------Interrupt-fr-die-Zeitbestimmung-der-einzelnen-Impulse-----------------------------------------------//
//-------------------------------------------------------------------------------------------------------------//

ISR( TIMER1_CAPT_vect )
{  
	Si_wert++;
    
		Endtime = ICR1;		        // Stopzeit fr die Messung= ICR1
		Durchlauf = TRUE;  			// Eine vollst?ndige Messung. Sie kann ausgewertet werden 
}

//------------------------------------------------------------------------------------------------------//
//------------------Interrupt-fr-Zhler-berlauf-------------------------------------------------------//
//------------------------------------------------------------------------------------------------------//	
ISR( TIMER1_OVF_vect )
{
	Overflow++;
}

//------------------------------------------------------------------------------------------------------//
//------------------Interrupt-fr-10-min.-erreicht------------------------------------------------------//
//------------------------------------------------------------------------------------------------------//	

ISR(TIMER1_COMPA_vect)
{
  sekunden++;    
}
//---------------------------------------------------------------------------------------------		  
//----------------Berechnung-der-zeit-zwischen-den-Impulsen------------------------------
//---------------------------------------------------------------------------------------------
void Impulse_berechnen(void)
{
	Zeit_ti = (Overflow * 65536) + Endtime - Starttime;
		
	Starttime = Endtime;
	Overflow=0;
	
	Zeit1_ti =(Zeit_ti*1000)/65536; 	//Zeit_ti in sekunden   
	
	ti_neu 	= ti_neu  + Zeit1_ti;
	Xi = Ct/ Zeit1_ti;         	//Geschwindigkeit von einzelnen Intervalle berechnen
	Xi1= Xi1 + Xi;				//Summe aller Geschwindigkeiten
	Xi2= Xi2 + (Xi * Xi);		//Summe aller Geschwindigkeiten hoch 2
	
	//my_uitoa(Zeit2_ti,lcd_Geschwindigkeit_String); //LCD String
		
}
//-------------------------------------------------------------------------------------------------------//
//----------------------Interrupts-Einschalten-----------------------------------------------------------//	
//-------------------------------------------------------------------------------------------------------//
void init_interrupt(void)			
{
	TCCR1B =  (1<<ICES1)|(1<<CS10)|(1<<CS11);		// Input Capture steigende Flanke an ICP-Pin(PD6), Prescale 64 
	TIMSK  = (1<<TICIE1)|(1<<TOIE1)|(1<<OCIE1A) ;	// Interrupts akivieren, Capture, Overflow 
	OCR1A  = 62499;
	sei();											// Interrups freigeben
}

//-------------------------------------------------------------------------------------------------------//
//----------------------Messwerte auf einem LCD ausgeben-------------------------------------------------//	
//-------------------------------------------------------------------------------------------------------//

void my_uitoa(uint32_t zahl, char* string) 
{
	int8_t i;                             // schleifenzhler
 
	string[10]='\0';                       // String Terminator
	for(i=9; i>=0; i--) 
	{
		string[i]=(zahl % 10) +'0';         // Modulo rechnen, dann den ASCII-Code von '0' addieren
		zahl /= 10;
	}
}

void my_print_LCD(char* string1, uint8_t start, uint8_t komma, uint8_t frac) 
{ 
  uint8_t i;            // Zhler
  uint8_t flag=0;       // Merker fr fhrende Nullen
 
  lcd_setcursor( 0,2 );				
  // Vorzeichen ausgeben  
  if (string1[0]=='-') lcd_data('-'); else lcd_data(' ');
 
  // Vorkommastellen ohne fhrende Nullen ausgeben
  for(i=start; i<komma; i++) 
  {
    if (flag==1 || string1[i]!='0') 
	{
		lcd_data(string1[i]);
		flag = 1;
    }
    else lcd_data(' ');         // Leerzeichen
  }
 
  lcd_data(',');                // Komma ausgeben
 
  // Nachkommastellen ausgeben
  for(; i<(komma+frac); i++) lcd_data(string1[i]);

  _delay_ms(200);

  //lcd_string("m/s");
  
  /*lcd_data('m');
  lcd_data('/');
  lcd_data('s');*/
}

void LCD_Messwerte_Anzeigen(void)
{
    	//Windgeschwindigkeit auf dem Display ausgeben
	ultoa( sekunden, lcd_sekunden_String, 10 );	// Das Ergebnis fuer die Anzeige aufbereiten 	  
	ultoa( Si_wert, lcd_Impulse_String, 10 ); 
	ultoa( Overflow, lcd_Overflow_String, 10 ); 
	lcd_clear();  
		
	lcd_setcursor( 0,1 );							// Gehe zu Pos. XY
	lcd_string(lcd_sekunden_String);			// Datenwert
	lcd_string("s");

	lcd_setcursor( 3,1 );				       			// Ausgabe Maske
	lcd_string(lcd_Impulse_String);			// Datenwert
	lcd_string("i");				       			// Ausgabe Maske

	lcd_setcursor( 6,1 );				       			// Ausgabe Maske
	lcd_string(lcd_Overflow_String);			// Datenwert
	lcd_string("d");	
	
	my_print_LCD(lcd_Geschwindigkeit_String,5,7,5);
	my_uitoa(ti_neu,lcd_Geschwindigkeit_String); //LCD String
	my_uitoa(Standardabweichung,lcd_Standardabweichung_String);
}


//-------------------------------------------------------------------------------------------------------//
//-----------------------Hauptprogram--------------------------------------------------------------------//	
//-------------------------------------------------------------------------------------------------------//	
		
int main (void)
{
	lcd_init();			     		//lcd initialisieren
	init_interrupt();
					//Interrups organisieren	

	while(1)					//diese Schleife soll das Programm immer wiederholen.
	{
		LCD_Messwerte_Anzeigen();
		
		if( Durchlauf == TRUE)  	// liegt ein vollstndiger Durchlauf vor?
		{
			Impulse_berechnen();
			Durchlauf = FALSE;
		}
			
		if (sekunden == Ges_Zeit)
		{
			if (Si_wert == 0)
			{
				Geschwindigkeit_in_10min = 0;
				Standardabweichung = 0;
			}
			else
			{
				Si_wert++;	
				ti_rest = (Ges_Zeit*1000)  - ti_neu; 
				ti_neu = ti_neu + ti_rest;
				Xi = Ct/ ti_rest;
				Xi1= Xi1 + Xi;
				Xi2= Xi2 + (Xi*Xi);
				
				Geschwindigkeit_in_10min = Xi1 / Si_wert;
				Standardabweichung = Xi2 - ((Xi1 * Xi1)/ Si_wert);
			
			//my_uitoa(Geschwindigkeit_in_10min,lcd_Geschwindigkeit_String);
			//my_uitoa(Standardabweichung,lcd_Standardabweichung_String);
				
				Starttime	= 0;
				Endtime  	= 0;
				Si_wert 	= 0;				// Si_wert  null setzen
				ti2_neu  	= 0;				// ti_neu  null setzen
				ti1_neu  	= 0;
				ti_neu   	= 0;				// ti_neu	null setzen	
				ti_rest		= 0;
				sekunden 	= 0;			    // 10 minuten zurcksetzen
				Xi 			= 0;
				Xi1			= 0;
				Xi2			= 0;
			
			}
		
		}
	}	
}	
